/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.yoko.bindings.corba;

import java.io.IOException;
import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.WSDLException;

import org.apache.yoko.bindings.corba.extensions.CorbaAddressExtensionHelper;
import org.apache.yoko.bindings.corba.extensions.CorbaBindingExtensionHelper;
import org.apache.yoko.bindings.corba.extensions.CorbaOperationExtensionHelper;
import org.apache.yoko.bindings.corba.extensions.CorbaTypesExtensionHelper;

import org.objectweb.celtix.Bus;
import org.objectweb.celtix.bindings.BindingFactory;
import org.objectweb.celtix.bindings.ClientBinding;
import org.objectweb.celtix.bindings.ServerBinding;
import org.objectweb.celtix.bindings.ServerBindingEndpointCallback;
import org.objectweb.celtix.ws.addressing.EndpointReferenceType;

public class CorbaBindingFactory implements BindingFactory {

    private Bus bus;

    public CorbaBindingFactory() {
    }

    public void init(Bus b) {
        bus = b;

        CorbaAddressExtensionHelper.addExtension(
                bus.getWSDLManager().getExtenstionRegistry(), Port.class, "address");
        CorbaBindingExtensionHelper.addExtension(
                bus.getWSDLManager().getExtenstionRegistry(), Binding.class, "binding");
        CorbaOperationExtensionHelper.addExtension(
                bus.getWSDLManager().getExtenstionRegistry(), BindingOperation.class, "operation");
        CorbaTypesExtensionHelper.addExtension(
                bus.getWSDLManager().getExtenstionRegistry(), Definition.class, "typeMapping");
    }

    public ClientBinding createClientBinding(EndpointReferenceType reference)
        throws WSDLException, IOException {
        return new CorbaClientBinding(bus, reference);
    }

    public ServerBinding createServerBinding(EndpointReferenceType reference,
                                             ServerBindingEndpointCallback cbFactory) 
        throws WSDLException, IOException {
        return new CorbaServerBinding(bus, reference, cbFactory);
    }
}
