/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.yoko.bindings.corba;

import javax.xml.ws.Binding;

import junit.framework.TestCase;

import org.apache.yoko.wsdl.CorbaConstants;

import org.objectweb.celtix.Bus;
import org.objectweb.celtix.bindings.BindingFactory;
import org.objectweb.celtix.bindings.BindingManager;
import org.objectweb.celtix.bindings.ClientBinding;
import org.objectweb.celtix.bindings.ServerBinding;
import org.objectweb.celtix.ws.addressing.EndpointReferenceType;

public class CorbaBindingFactoryTest extends TestCase {

    private TestUtils testUtils;
    
    public CorbaBindingFactoryTest(String arg0) {
        super(arg0);
    }
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(CorbaBindingFactoryTest.class);
    }
    
    protected void setUp() throws Exception {
        super.setUp();
        
        testUtils = new TestUtils();
    }

    public void testCreateClientBinding() throws Exception {
        Bus bus = Bus.init(new String[0]);
        
        // programmatically add the corba binding factory
        BindingManager manager = bus.getBindingManager();
        CorbaBindingFactory corbaBF = new CorbaBindingFactory();
        corbaBF.init(bus);
        manager.registerBinding(CorbaConstants.NU_WSDL_CORBA, corbaBF);
        
        BindingFactory factory = 
            bus.getBindingManager().getBindingFactory("http://schemas.apache.org/yoko/bindings/corba");
        
        EndpointReferenceType address = testUtils.getSimpleTestEndpointReference();
        
        ClientBinding cb = factory.createClientBinding(address);
        assertNotNull(cb);
        assertTrue(CorbaClientBinding.class.isInstance(cb));
        
        CorbaClientBinding clientBinding = (CorbaClientBinding)cb;
        Binding binding = clientBinding.getBinding();
        assertNotNull(binding);
        assertTrue(CorbaBindingImpl.class.isInstance(binding));
    }
    
    public void testCreateServerBinding() throws Exception {
        Bus bus = Bus.init(new String[0]);
        
        // programmatically add the corba binding factory
        BindingManager manager = bus.getBindingManager();
        CorbaBindingFactory corbaBF = new CorbaBindingFactory();
        corbaBF.init(bus);
        manager.registerBinding(CorbaConstants.NU_WSDL_CORBA, corbaBF);
        
        BindingFactory factory = 
            bus.getBindingManager().getBindingFactory("http://schemas.apache.org/yoko/bindings/corba");
        
        EndpointReferenceType address = testUtils.getSimpleTestEndpointReference();
        
        ServerBinding sb = factory.createServerBinding(address, null);
        assertNotNull(sb);
        assertTrue(CorbaServerBinding.class.isInstance(sb));
        
        CorbaServerBinding serverBinding = (CorbaServerBinding)sb;
        Binding binding = serverBinding.getBinding();
        assertNotNull(binding);
        assertTrue(CorbaBindingImpl.class.isInstance(binding));
    }
}
