/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.wsdl.Definition;
import javax.wsdl.Message;

import javax.xml.namespace.QName;

import antlr.ASTVisitor;
import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.ArgType;
import org.apache.schemas.yoko.bindings.corba.OperationType;

import org.apache.ws.commons.schema.XmlSchemaElement;
import org.apache.ws.commons.schema.XmlSchemaSequence;

public class ReturnParameterVisitor implements ASTVisitor {

    static final String RETURN_PARAMETER = "return";
    static final String OUT_PARAMETER = "outparameter";
    
    Definition definition;
    Message outputMsg;
    OperationType corbaOperation;
    TypesVisitor typesVisitor;

    XmlSchemaSequence outWrappingSequence;
    XmlSchemaElement outWrappedElement;

    public ReturnParameterVisitor(WSDLASTVisitor visitor,
                                  XmlSchemaSequence outWrapSeq) {
        definition = visitor.getDefinition();   
        typesVisitor = visitor.getTypesVisitor();
        outWrappingSequence = outWrapSeq;
    }

    public void setOutputMessage(Message msg) {
        outputMsg = msg;
    }

    public Message getOutputMessage() {
        return outputMsg;
    }

    public void setCorbaOperation(OperationType op) {
        corbaOperation = op;
    }

    private void createOutputParameter(String partName, ArgType param, AST typeNode) {
        QName element = outputMsg.getQName();
        outWrappedElement = typesVisitor.addElement(outWrappingSequence, element, partName);
        typesVisitor.setCurrentPart(outWrappedElement, partName, param);
        typesVisitor.visit(typeNode);
    }
   
    public void visit(AST node) {
        String partName = RETURN_PARAMETER;
        ArgType param = createCorbaReturn(partName);
        createOutputParameter(partName, param, node);        
    }
    
    private ArgType createCorbaReturn(String partName) {
        ArgType param = new ArgType();
        param.setName(partName);
        corbaOperation.setReturn(param);
        return param;
    }
}

